#include <chrono>
#include <iostream>
#include <vector>
#include <random>

#ifdef VECTOR
#include "vector.h"
#else
#include "scalar.h"
#endif

class Timer {
    using TimePoint = std::chrono::time_point < std::chrono::system_clock >;
    TimePoint _startTime;
    TimePoint _stopTime;
public:
    void start() {
        _startTime = now();
    }

    void stop() {
        _stopTime = now();
    }

    TimePoint now() {
        return std::chrono::system_clock::now();
    }

    long long elapsed() const {
        return std::chrono::duration_cast< std::chrono::milliseconds >( _stopTime - _startTime ).count();
    }
};

struct Stat {
    using Type = Average::ResultType;
    Type current;
    Type all;

    void set( Type c, Type a ) {
        current = c;
        all = a;
    }
};

int main( int argc, char **argv ) {
    Average cur, all;
    const int bsize = 32768;
    const int bcount = 4096;

    Average::Data data[bcount];
    Stat stats[bcount];

    std::random_device rd;
    std::mt19937 generator(rd());
    std::uniform_int_distribution<> distribution(0, 4096 - 1);

    Timer timer;

    for ( auto &row : data ) {
        Average::init( row, bsize );
        for ( int i = 0; i < bsize; ++i )
            row[ i ] = distribution( generator );
    }

    timer.start();
    for ( int i = 0; i < bcount; ++i ) {
        cur.reset();
        cur.process( data[i] );
        all.process( data[i] );
        stats[ i ].set( cur.average(), all.average() );
    }
    timer.stop();

    for ( int i = 0; i < bcount; ++i ) {
        std::cerr << "block " << i << ": " << stats[ i ].current
                  << ", running: " << stats[ i ].all << std::endl;
    }

    std::cerr << "done in " << timer.elapsed() << "ms" << std::endl;

    return 0;
}
